
import "Turbine.Gameplay";
import "Turbine.UI.Extensions";

WEQuickslotWindow = class( Turbine.UI.Window );
--saferun( [[ --
function WEQuickslotWindow:Constructor(builderType, settings)
	Turbine.UI.Window.Constructor( self );

	self.settings = settings;
	self:SetPosition( self.settings.left, self.settings.top );
	self.gambitBuilder = builderType;
	self.spacing = 2;

	self.quickslotSize = 32;
	self:UpdateDescriptionWidth();
	self.descriptions = { };
	self.gambitControls = { };
	self.quickslots = { };

	self.player = Turbine.Gameplay.LocalPlayer.GetInstance();
	self.player.LevelChanged = function( sender, args )
		self:RefreshQuickslots();
	end

	self.PositionChanged = function( sender, args )
		self.settings.left, self.settings.top = self:GetPosition();
		self:SaveSettings();
	end

	self:RefreshQuickslots();

	self.DragBar = Deusdictum.UI.DragBar( self, self.gambitBuilder );
	self.DragBar:SetRotator( true );
	
	self:SetVisible( true );
	self:SetOpacity( 1 );
end

function WEQuickslotWindow:UpdateDescriptionWidth()
	if self.settings.transposed or not self.settings.descriptionsOn then
		self.descriptionWidth = 0;
	else
		self.descriptionWidth = 204;
	end
end

function WEQuickslotWindow:Rotate( args )
	self.settings.transposed = not self.settings.transposed;
	self:UpdateDescriptionWidth();
	if not self.settings.transposed then
		for i = 1, #(self.gambitControls), 1 do
			self.gambitControls[i]:SetSize( self.quickslotSize + self.descriptionWidth, self.quickslotSize );
			self:CreateHint( self.gambits[i], self.gambitControls[i] );
		end
	end
	self:SaveSettings();
	self:RefreshQuickslots();
	self.DragBar:RecalculateSize();
end

function WEQuickslotWindow:SaveSettings()
	Deusdictum.Plugins.WardenEase.WEmain:SaveSettings();
end

function WEQuickslotWindow:RefreshQuickslots()
	local i, k, v;
	local lvl = self.player:GetLevel();
	local descriptionsOn = self.settings.descriptionsOn;

	if self.settings.transposed then
		descriptionsOn = false;
	end

	self.gambits = {  };

	for k,v in pairs(Deusdictum.Data.Warden.Gambits) do
		if (v.Sequence[1].Builds == self.gambitBuilder and lvl >= v.Level) then
--		if (v.Sequence[1].Builds == self.gambitBuilder) then
			table.insert(self.gambits, v);
		end
	end

	-- Remove any quickslots no longer required.
	for i = #(self.gambits) + 1, #(self.quickslots), 1 do
		self.quickslots[#(self.gambits) + 1]:SetVisible( false );
		self.quickslots[#(self.gambits) + 1]:SetParent( nil );
		table.remove(self.quickslots, #(self.gambits) + 1);
		self.gambitControls[#(self.gambits) + 1]:SetVisible( false );
		self.gambitControls[#(self.gambits) + 1]:SetParent( nil );
		table.remove(self.gambitControls, #(self.gambits) + 1);
	end
	
	-- Add any new quickslots that should be created.
	for i = #(self.quickslots) + 1, #(self.gambits), 1 do
		self.quickslots[i] = Turbine.UI.Control();
--		self.quickslots[i] = Turbine.UI.Lotro.Quickslot();
		self.quickslots[i]:SetSize( self.quickslotSize, self.quickslotSize );
		self.quickslots[i]:SetAllowDrop( false );
--		self.quickslots[i]:SetShortcut( Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Alias, "/we " .. self.gambits[i].Key ));
		self.quickslots[i]:SetBackground(self.gambits[i].Resource);
		self.quickslots[i].gambit = self.gambits[i];

		self.quickslots[i].MouseClick = function( sender, args )
			Deusdictum.Plugins.WardenEase.WEmain:SetGambit( sender.gambit.Sequence );
		end

		self.quickslots[i].MouseHover = function( sender, args )
			if ( not self.settings.descriptionsOn or self.settings.transposed ) and self.quickslots[i].tooltip == nil then
--				Turbine.Shell.WriteLine( "Hovering..." );
--				self.quickslots[i].tooltip = self:CreateHint( self.gambits[i] );
--				local left, top = self:PointToScreen( self:GetMousePosition() );

--				self.quickslots[i].tooltip:SetPosition( left - self:GetWidth(), top - self:GetHeight() );
			end
		end

		self.quickslots[i].MouseLeave = function( sender, args )
			self.quickslots[i].tooltip = nil;
		end

		self.gambitControls[i] = Turbine.UI.Control();
		self.gambitControls[i]:SetSize( self.quickslotSize + self.descriptionWidth, self.quickslotSize );
		self.gambitControls[i]:SetParent( self );
		self.gambitControls[i]:SetAllowDrop( false );
--		self.gambitControls[i]:SetBackColor( Turbine.UI.Color( 0.2, 0, 0, 0 ) );
		self.gambitControls[i]:SetVisible( true );
		self.gambitControls[i].gambit = self.gambits[i];

		self.quickslots[i]:SetParent( self.gambitControls[i] );
		self.quickslots[i]:SetPosition( 0, 0 );
		self.quickslots[i]:SetVisible( true );

		self:CreateHint( self.gambits[i], self.gambitControls[i] );
	end

	self.maxGambitLength = 0;

	for i = 1, #(self.quickslots), 1 do
		if #(self.quickslots[i].gambit.Sequence) > self.maxGambitLength then
			self.maxGambitLength = #(self.quickslots[i].gambit.Sequence);
		end
	end

	self:RefreshLayout();
end

function WEQuickslotWindow:CreateHint( gambit, parent )
	if type( parent ) == "table" then
		hint = Turbine.UI.Control();
		hint:SetParent( parent );
		hint:SetPosition( self.quickslotSize, 0 );
	else
		hint = Turbine.UI.Window();
--		hint:SetPosition(  );
	end

	hint:SetSize( self.descriptionWidth, self.quickslotSize );
	hint:SetAllowDrop( false );
	hint:SetBackColor( Turbine.UI.Color( 0.2, 0, 0, 0 ) );
	hint:SetVisible( true );

	local left = 3;
	local buf = "";

	if gambit.Targets > 1 then
		local curLabel = Turbine.UI.Label();
		curLabel:SetParent( hint );
		curLabel:SetPosition( left, 19);
		curLabel:SetFont( Turbine.UI.Lotro.Font.TrajanPro13 );
		curLabel:SetFontStyle( Turbine.UI.FontStyle.Outline );
		curLabel:SetOutlineColor( Turbine.UI.Color( 1, 0, 0, 0 ) );
		curLabel:SetForeColor( Turbine.UI.Color( 1, 0.5, 0.5, 0.5 ) );
		curLabel:SetText( "x" .. gambit.Targets );
		curLabel:SetBackColor( Turbine.UI.Color( 0.2, 0, 0, 0 ) );
		curLabel:SetVisible( true );

		left = left + ( curLabel:GetTextLength() + 1 ) * 7;
		buf = " ";
	end

	for j = 1, #(gambit.Effects), 1 do
		local curLabel = Turbine.UI.Label();
		curLabel:SetParent( hint );
		curLabel:SetPosition( left, 19);
		curLabel:SetFont( Turbine.UI.Lotro.Font.TrajanPro13 );
		curLabel:SetFontStyle( Turbine.UI.FontStyle.Outline );
		curLabel:SetOutlineColor( Turbine.UI.Color( 1, 0, 0, 0 ) );
		curLabel:SetForeColor( gambit.Effects[j].effect.color );
		if type(gambit.Effects[j].tier) == "number" then
			curLabel:SetText( buf .. gambit.Effects[j].effect.text .. gambit.Effects[j].tier );
		else
			curLabel:SetText( buf .. gambit.Effects[j].effect.text );
		end
		curLabel:SetBackColor( Turbine.UI.Color( 0.2, 0, 0, 0 ) );
		curLabel:SetVisible( true );

		left = left + ( curLabel:GetTextLength() + 1 - string.len(buf) ) * 7;
		buf = " ";
	end

	local left = 3;

	for j = 1, #(gambit.Sequence), 1 do
		local curControl = Turbine.UI.Control();
		curControl:SetParent( hint );
		curControl:SetPosition( left, 2);
		curControl:SetSize( 15, 15 );
		curControl:SetBackground( gambit.Sequence[j].SmallResource );
		curControl:SetVisible( true );

		left = left + 17;
	end

	local curLabel = Turbine.UI.Label();
	curLabel:SetParent( hint );
	curLabel:SetPosition( left, 4 );
	curLabel:SetSize( hint:GetWidth() - left, 15 );
	if gambit.String == "Dark Before The Dawn"
	 or gambit.String == "Exultation Of Battle" then
		curLabel:SetFont( Turbine.UI.Lotro.Font.Arial12 );
		curLabel:SetForeColor( Turbine.UI.Color( 1, 0.4, 0.4, 0.4 ) );
	else
		curLabel:SetFont( Turbine.UI.Lotro.Font.TrajanPro13 );
		curLabel:SetForeColor( Turbine.UI.Color( 1, 0.5, 0.5, 0.5 ) );
	end
	curLabel:SetFontStyle( Turbine.UI.FontStyle.Outline );
	curLabel:SetOutlineColor( Turbine.UI.Color( 1, 0, 0, 0 ) );
	curLabel:SetText( " " .. gambit.String );
	curLabel:SetBackColor( Turbine.UI.Color( 0.2, 0, 0, 0 ) );
	curLabel:SetVisible( true );

	return hint;
end

function WEQuickslotWindow:pvar(tmp)
	Turbine.Shell.WriteLine(tmp);
end

function WEQuickslotWindow:RefreshLayout()
	local displayWidth = Turbine.UI.Display:GetWidth();
	local displayHeight = Turbine.UI.Display:GetHeight();
	local descriptionsOn = self.settings.descriptionsOn;

	self.slotsWide = 1;
	self.slotsHigh = #(self.gambits);

	if self.settings.transposed then
		self.slotsWide, self.slotsHigh = self.slotsHigh, self.slotsWide;
		descriptionsOn = false;
	end

	local i, width, height;

	width = self.slotsWide * (self.spacing + self.quickslotSize) - self.spacing + self.descriptionWidth;
	height = self.slotsHigh * (self.spacing + self.quickslotSize);

	self:SetSize(width, height)

	for i = 1, #(self.gambits), 1 do
		local col, row;

		col = (i - 1) % self.slotsWide;
		row = math.floor((i - 1) / self.slotsWide);
		self.gambitControls[i]:SetPosition( col * (self.quickslotSize + self.spacing), row * (self.quickslotSize + self.spacing) );
	end
end
--]], "WEQuickslotWindow");